import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

public class Algorithm {
    public Algorithm(Graph g, int[][] adjacency_matrix, int numberofvertices,
                     Map<Integer, String> map) {
        System.out.println("\nAlgo de gestion des ressources mobiles rares dans un " +
                "Internet Physique");

        // Étape 1: Calcul du PCC entre tous les pairs de noeuds
        // Le resultat du calcul est stocké dans le tableau distancematrix
        int[][] distancematrix = new int[numberofvertices][numberofvertices];
        int[][] successorsMatrix = new int[numberofvertices][numberofvertices];
        FloydWarshall floydwarshall = new FloydWarshall(numberofvertices, distancematrix, successorsMatrix);
        floydwarshall.floydwarshall(adjacency_matrix, distancematrix, successorsMatrix);
        System.out.println("\nÉtape 1: Calcul du plus court chemin entre tous les " +
                "pairs de noeuds : \n");
        floydwarshall.printResult(map);

        // Étape 2: Calcule du PCC avec déviation entre tous les sommets
        // Le resultat du calcul est stocké dans le tableau pCCAvecDeviation_matrix
        int[][] pCCAvecDeviation_matrix;
        pCCAvecDeviation_matrix = new int[numberofvertices][numberofvertices];
        pCCAvecDeviation(numberofvertices, distancematrix, pCCAvecDeviation_matrix);
        System.out.println("\nÉtape 2: Calcul du Coût du plus court chemin avec déviation entre\n" +
                "toutes les paires de sommets : \n");
        afficheMatrice(pCCAvecDeviation_matrix, map);

        // Étape 3.1 : formulation d'un problème de transport
        // Le resultat sera stocké dans le fichier logistiqua.txt
        System.out.println("\nÉtape 3.1 : Extraction du problème de transport à partir des informations " +
                "fournies avec le graphe : \n");
        List<Node> src = new ArrayList<>();
        List<Node> dst = new ArrayList<>();
        int[][] transPb_matrix = null;
        try {
            transPb_matrix = extractTheTransPbFromTheGraph(g, pCCAvecDeviation_matrix, src, dst);
        } catch (Exception e) {
            System.out.println("Erreur de fichier");
            System.exit(0);
        }

        // Étape 3.2 : Stepping-Stone pour résoudre le problème de transport ainsi obtenu
        TransportationProblem transportationProblem = new TransportationProblem();
        String filename = "logistiqua.txt";
        transportationProblem.init(filename);
        transportationProblem.northWestCornerRule();
        transportationProblem.steppingStone();
        System.out.println("\nÉtape 3.2 : Stepping-Stone pour résoudre le problème de transport ainsi obtenu : \n");
        System.out.println("Les lignes répresentent les noeuds sources et les colonnes les noeuds destinations.\n" +
                "À l'intersection des deux, le nombre de conteneurs refeers à envoyer : \n");
        int[][] transPbSolution = transportationProblem.printResult(src, dst);

        // Étape 4 : trie des trajets par ordre croissant suivant le
        // coût de la déviation de trajectoire
        System.out.println("\nDernière étape : Liste des trajets (des noeuds sources aux noeuds destinations)\n" +
                "trié par ordre croissant suivant le coût de la déviation de trajectoire : \n");
        List<Trajet> trajets = trieTrajets(g, transPb_matrix, transPbSolution, src, dst, floydwarshall, map);
        trajets.forEach(System.out::println);
    }

    private static void pCCAvecDeviation(int numberofvertices, int[][] distanceMatrix, int[][] deviationCost_matrix) {
        for (int source = 0; source < numberofvertices; source++) {
            for (int destination = 0; destination < numberofvertices; destination++) {
                deviationCost_matrix[source][destination] = distanceMatrix[0][source]
                        + distanceMatrix[source][destination]
                        + distanceMatrix[destination][numberofvertices - 1]
                        - distanceMatrix[0][numberofvertices - 1];
            }
        }
    }

    private static void afficheMatrice(int[][] m, Map<Integer, String> map) {
        for (int source = 0; source < m.length; source++)
            System.out.print("\t" + map.get(source));
        System.out.println();

        for (int source = 0; source < m.length; source++) {
            System.out.print(map.get(source) + "\t");
            for (int destination = 0; destination < m[source].length; destination++) {
                System.out.print(m[source][destination] + "\t");
            }
            System.out.println();
        }
    }

    private int[][] extractTheTransPbFromTheGraph(Graph g, int[][] pCCAvecDeviation_matrix, List<Node> src,
                                                  List<Node> dst) throws IOException {
        for (int i = 0; i < g.getNodeCount(); i++) {
            Node n = g.getNode(i);
            if (n.getNumber("offre") > n.getNumber("demande")) {
                src.add(n);
            } else {
                dst.add(n);
            }
        }
        System.out.println("Liste des noeuds sources (ayant un surplus de conteneur reefers) : " + src);
        System.out.println("Liste des noeuds destinations (ayant un besoin de conteneur reefers) : " + dst);

        FileWriter fw = new FileWriter("logistiqua.txt");
        BufferedWriter bw = new BufferedWriter(fw);
        bw.write(src.size() + " " + dst.size() + "\n");

        for (Node node : src) bw.write((int) node.getNumber("offre") + " ");
        bw.write("\n");
        for (Node node : dst) bw.write((int) node.getNumber("demande") + " ");
        bw.write("\n");

        System.out.println("\nCoût du plus court chemin avec déviation entre les noeuds\n" +
                "sources (lignes) et les noeuds destinations (colonnes) : \n");
        for (Node node : dst) {
            System.out.print("\t" + node.getId());
        }
        System.out.println();
        int[][] transPb_matrix = new int[src.size()][dst.size()];
        for (int i = 0; i < src.size(); i++) {
            Node nodeSrc = src.get(i);
            System.out.print(nodeSrc.getId() + "\t");
            for (int j = 0; j < dst.size(); j++) {
                Node nodeDst = dst.get(j);
                transPb_matrix[i][j] = pCCAvecDeviation_matrix[nodeSrc.getIndex()][nodeDst.getIndex()];
                bw.write(transPb_matrix[i][j] + " ");
                System.out.print(transPb_matrix[i][j] + "\t");
            }
            bw.write("\n");
            System.out.println();
        }
        bw.close();
        return transPb_matrix;
    }

    private List<Trajet> trieTrajets(Graph g, int[][] transPb_matrix, int[][] transPbSolutionMatrix,
                                     List<Node> src, List<Node> dst, FloydWarshall floydWarshall, Map<Integer, String> map) {
        List<Trajet> trajets = new ArrayList<>();

        for (int i = 0; i < transPb_matrix.length; i++) {
            for (int j = 0; j < transPb_matrix[i].length; j++) {
                if (transPbSolutionMatrix[i][j] != 0) {
                    trajets.add(new Trajet(src.get(i), dst.get(j), transPbSolutionMatrix[i][j], transPb_matrix[i][j],
                            floydWarshall.returnPath(g, map, src.get(i).getIndex(), dst.get(j).getIndex())));
                }
            }
        }
        trajets.sort(Comparator.comparing(Trajet::getDeviationCost));
        return trajets;
    }

    private static class Trajet {
        Node source, destination;
        int nbContToSend, deviationCost;
        List<Node> pcc;

        public Trajet(Node source, Node destination, int nbContToSend, int deviationCost, List<Node> pcc) {
            this.source = source;
            this.destination = destination;
            this.nbContToSend = nbContToSend;
            this.deviationCost = deviationCost;
            this.pcc = pcc;
        }

        public Node getSource() {
            return source;
        }

        public Node getDestination() {
            return destination;
        }

        public int getNbContToSend() {
            return nbContToSend;
        }

        public int getDeviationCost() {
            return deviationCost;
        }

        public List<Node> getPcc() {
            return pcc;
        }

        public String toString() {
            return source.getId() + " -> " + destination.getId() + " (Nombre de conteneurs " +
                    "à envoyer = " + nbContToSend + ", Coût de la déviation = "
                    + deviationCost + ", Plus court chemin = " + pcc + ")";
        }
    }
}
