import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static java.lang.String.format;

public class FloydWarshall {
    public static final int INFINITY = 999;
    private final int[][] distancematrix, successorsMatrix;
    private final int numberofvertices;

    public FloydWarshall(int numberofvertices, int[][] distancematrix, int[][] successorsMatrix) {
        this.distancematrix = distancematrix;
        this.successorsMatrix = successorsMatrix;
        this.numberofvertices = numberofvertices;

        // initialisation de la matrice des successeurs
        for (int i = 0; i < successorsMatrix.length; i++) {
            for (int j = 0; j < successorsMatrix.length; j++)
                if (i != j)
                    successorsMatrix[i][j] = j + 1;
        }
    }

    public void floydwarshall(int[][] adjacencymatrix, int[][] distancematrix, int[][] successorsMatrix) {
        for (int source = 0; source < numberofvertices; source++) {
            System.arraycopy(adjacencymatrix[source], 0, distancematrix[source], 0, numberofvertices);
        }

        for (int intermediate = 0; intermediate < numberofvertices; intermediate++) {
            for (int source = 0; source < numberofvertices; source++) {
                for (int destination = 0; destination < numberofvertices; destination++) {
                    if (distancematrix[source][intermediate] + distancematrix[intermediate][destination] <
                            distancematrix[source][destination]) {

                        distancematrix[source][destination] = distancematrix[source][intermediate] +
                                distancematrix[intermediate][destination];

                        successorsMatrix[source][destination] = successorsMatrix[source][intermediate];
                    }
                }
            }
        }
    }

    public void printResult(Map<Integer, String> map) {
        System.out.println("pair     dist    path");
        for (int i = 0; i < successorsMatrix.length; i++) {
            for (int j = 0; j < successorsMatrix.length; j++) {
                if (i != j) {
                    int u = i + 1;
                    int v = j + 1;
                    StringBuilder path = new StringBuilder(format("%s -> %s    %2d     %s",
                            map.get(u - 1), map.get(v - 1), distancematrix[i][j], map.get(u - 1)));
                    do {
                        u = successorsMatrix[u - 1][v - 1];
                        path.append(" -> ").append(map.get(u - 1));
                    } while (u != v);
                    System.out.println(path);
                }
            }
        }
    }

    public List<Node> returnPath(Graph g, Map<Integer, String> map, int i, int j) {
        List<Node> path = new ArrayList<>();
        if (i != j) {
            int u = i + 1;
            int v = j + 1;
            path.add(g.getNode(map.get(u - 1)));
            do {
                u = successorsMatrix[u - 1][v - 1];
                path.add(g.getNode(map.get(u - 1)));
            } while (u != v);
        }
        return path;
    }

}