import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;

import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileSystemView;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

public class Main {
    public static void main(String... arg) {
        //  Modélisation du problème
        Graph g = new SingleGraph("g");
        try {
            g.read(openFile().getPath());
        } catch (Exception e) {
            System.out.println("Erreur de fichier"); System.exit(0);
        }
        int[][] adjacency_matrix;
        Map<Integer, String> map = new HashMap<>();
        int numberofvertices = g.getNodeCount();
        adjacency_matrix = new int[numberofvertices][numberofvertices];
        initMatrices(g, adjacency_matrix, numberofvertices, map);
        displayModel(g);
        // Démarrage de l'algorithme
        new Algorithm(g, adjacency_matrix, numberofvertices, map);
    }

    private static File openFile() {
        JFileChooser choose = new JFileChooser(
                FileSystemView
                        .getFileSystemView()
                        .getHomeDirectory()
        );

        choose.setDialogTitle("Selectionnez le fichier dgs");
        choose.setAcceptAllFileFilterUsed(false);
        FileNameExtensionFilter filter = new FileNameExtensionFilter("Graphe au format dgs", "dgs");
        choose.addChoosableFileFilter(filter);
        choose.showOpenDialog(null);
        return choose.getSelectedFile();
    }

    private static void initMatrices(Graph g, int[][] adjacency_matrix, int numberofvertices,
                                     Map<Integer, String> map) {
        for (int source = 0; source < numberofvertices; source++) {
            map.put(source, g.getNode(source).getId());
            for (int destination = 0; destination < numberofvertices; destination++) {
                Edge e = g.getNode(source).getEdgeBetween(destination);
                adjacency_matrix[source][destination] = (e != null) ? (int) e.getNumber("dist") : 0;
                if (source == destination) {
                    adjacency_matrix[source][destination] = 0;
                    continue;
                }
                if (adjacency_matrix[source][destination] == 0) {
                    adjacency_matrix[source][destination] = FloydWarshall.INFINITY;
                }
            }
        }
    }

    private static void displayModel(Graph g) {
        System.setProperty("org.graphstream.ui", "swing");
        for (Node n : g) {
            n.setAttribute("ui.label", n.getId() + " (x=" + (int) n.getNumber("offre")
                    + "; y=" + (int) n.getNumber("demande") + ")");
            n.setAttribute("ui.style", "shape: box; size: 75px,25px; fill-color: yellow;");
        }
        for (int i = 0; i < g.getEdgeCount(); i++) {
            Edge e = g.getEdge(i);
            e.setAttribute("ui.label", e.getNumber("dist"));
            e.setAttribute("ui.style", "text-alignment: under;");
        }
        g.display();
    }

}
